import os
import signal

import click
import rich
from InquirerPy import inquirer
from requests import ConnectTimeout

from abst.__version__ import __version_name__, __version__
from abst.bastion_support.bastion_scheduler import BastionScheduler
from abst.bastion_support.oci_bastion import Bastion
from abst.cli_commands.config_cli.commands import config
from abst.cli_commands.context.commands import context
from abst.cli_commands.cp_cli.commands import cp
from abst.cli_commands.create_cli.commands import create
from abst.cli_commands.helm_cli.commands import helm
from abst.cli_commands.kubectl_cli.commands import ssh_pod, log_pod
from abst.cli_commands.parallel.commands import parallel
from abst.config import default_creds_path, default_contexts_location, default_conf_path
from abst.notifier.version_notifier import Notifier
from abst.utils.misc_funcs import setup_calls


@click.group()
@click.version_option(f"{__version__} {__version_name__}")
def cli():
    pass


@cli.command(
    "use",
    help="Will Switch context to be used,"
         " use default for default context specified"
         " in creds.json",
)
@click.option("--debug", is_flag=True, default=False)
@click.argument("context-name", default="")
def use(debug, context_name):
    setup_calls(debug)

    used_context = context_name
    if context_name == "":
        contexts = BastionScheduler.get_contexts()
        if len(contexts) == 0:
            rich.print("No Contexts available, selected default")
            used_context = "default"
        else:
            contexts.insert(0, "default")
            used_context = inquirer.select("Select context to use:", contexts).execute()

    Bastion.create_default_locations()
    conf = Bastion.load_config()
    conf["used_context"] = None if used_context == "default" else used_context

    rich.print("[green]Successfully Switched[/green]")
    rich.print(
        f"[white]Currently used context is[/white] [green]{used_context}[/green] "
        f"[gray](This does not change context in .kube)[/gray]"
    )
    Bastion.write_creds_json(conf, default_conf_path)


@cli.command("clean", help="Cleans all credentials created by abst")
def clean():
    """ """
    file = "not specified"
    files = [*default_contexts_location.iterdir()]
    if default_creds_path.exists():
        files.append(default_creds_path)
    try:
        confirm = inquirer.confirm(
            "Do you really want to Delete all Creds file ? All of credentials will be "
            "lost"
        ).execute()
        if not confirm:
            rich.print("[green]Cancelling, nothing changed[/green]")
            exit(0)

        rich.print("[red]Deleting all of the credentials[/red]")
        for file in files:
            os.remove(str(file))
    except PermissionError:
        print("Do not have permission to remove, or process is using this file.")
        print(f"Please delete manually in {file}")


def main():
    try:
        Notifier.notify()
    except (ConnectionError, ConnectTimeout):
        return False
    Bastion.create_default_locations()
    cli()


signal.signal(signal.SIGINT, BastionScheduler.kill_all)
signal.signal(signal.SIGTERM, BastionScheduler.kill_all)
signal.signal(signal.SIGABRT, BastionScheduler.kill_all)

cli.add_command(parallel)
cli.add_command(config)
cli.add_command(context)
cli.add_command(helm)
cli.add_command(cp)
cli.add_command(ssh_pod)
cli.add_command(log_pod)
cli.add_command(create)

if __name__ == "__main__":
    main()



from datetime import datetime

import lastversion
import requests
import rich
import semantic_version
from requests import ConnectTimeout

from abst import __version__
from abst.config import default_conf_path


class Notifier:
    @classmethod
    def check_pypi_available(cls):
        try:
            req = requests.get("https://pypi.org/", timeout=0.2)
            return req.status_code == 200
        except (ConnectionError, ConnectTimeout):
            return False

    @classmethod
    def get_last_version(cls):
        return lastversion.latest(__version__.__pypi_repo__)

    @classmethod
    def is_last_version(cls):
        last = cls.get_last_version()
        return semantic_version.Version(
            __version__.__version__) >= semantic_version.Version(str(last))

    @classmethod
    def notify(cls):
        from abst.bastion_support.oci_bastion import Bastion
        cfg = Bastion.load_config()
        dt = datetime.now()
        if (dt - datetime.fromtimestamp(
                cfg.get("last-check", 1677884000))).seconds < 60 * 60 * 8:
            return

        cfg["last-check"] = datetime.timestamp(dt)
        Bastion.write_creds_json(cfg, default_conf_path)

        if cls.check_pypi_available() and not cls.is_last_version():
            last = cls.get_last_version()
            rich.print(
                f"[yellow]WARNING: You are using abst version {__version__.__version__}; however,"
                f" version {last} is available.[/yellow]")
            rich.print(
                f"[yellow]You should consider upgrading via the `[green]pip3 install abst --upgrade[/green]`"
                f" command.[/yellow]")



from pathlib import Path
from typing import Optional

from abst.bastion_support.bastion_scheduler import BastionScheduler
from abst.bastion_support.oci_bastion import Bastion
from abst.config import default_creds_path, default_contexts_location


def get_context_path(context_name):
    if context_name is None:
        return default_creds_path
    else:
        return default_contexts_location / (context_name + ".json")


def display_scheduled(set_dir: Optional[Path] = None):
    from rich.table import Table
    from rich.console import Console
    console = Console()
    table = Table(title=f"Bastions of {set_dir.name} set" if set_dir else "Bastions of default stack", highlight=True)

    table.add_column("Name", justify="left", style="cyan", no_wrap=True)
    table.add_column("Local Port", style="magenta", no_wrap=True)
    table.add_column("Active", justify="right", style="green", no_wrap=True)
    table.add_column("Status", justify="right", style="green", no_wrap=True)
    if set_dir:
        for context_path in filter(lambda p: not str(p.name).startswith("."), set_dir.iterdir()):
            conf = Bastion.load_json(context_path)
            context_name = context_path.name[:-5]
            table.add_row(context_name, conf.get('local-port', 'Not Specified'),
                          conf.get('target-ip', 'Not Specified'),
                          conf.get('target-port', 'Not Specified'))
    else:
        for context_path in BastionScheduler.get_bastions():
            conf = Bastion.load_json(Bastion.get_creds_path_resolve(context_path))
            table.add_row(context_path, conf.get('local-port', 'Not Specified'),
                          conf.get('target-ip', 'Not Specified'),
                          conf.get('target-port', 'Not Specified'))
    console.print(table)



import json
import logging
from pathlib import Path
from typing import Optional

import rich
from rich.logging import RichHandler

from abst.bastion_support.oci_bastion import Bastion
from abst.config import default_contexts_location


def setup_calls(debug):
    setup_debug(debug)


def setup_debug(debug):
    if not debug:
        logging.disable(logging.DEBUG)
        logging.disable(logging.INFO)
    from imp import reload  # python 2.x don't need to import reload, use it directly
    reload(logging)
    logging.basicConfig(
        level=logging.DEBUG if debug else logging.CRITICAL,
        format="%(message)s",
        datefmt="[%X]",
        handlers=[RichHandler()],
    )


def print_eligible(searched: str):
    contexts = Bastion.get_contexts()
    rich.print("[bold]Configured contexts:[/bold]")
    for key, context in contexts.items():
        do_skip = False
        for value in context.values():
            if searched in value:
                do_skip = True
                break
        if do_skip:
            continue
        rich.print(key)


def get_context_data(name) -> Optional[dict]:
    if name in [file.name.replace(".json", "") for file in
                Path(default_contexts_location).iterdir()]:
        rich.print("[bold]Context config contents:[/bold]\n")
        with open(Path(default_contexts_location) / (name + ".json"), "r") as f:
            data = json.load(f)
            return data
    else:
        rich.print("[red]Context does not exists[/red]")
        return None



import json
from json import JSONDecodeError
from pathlib import Path

import rich
from InquirerPy import inquirer

from abst.bastion_support.oci_bastion import Bastion
from abst.config import default_config_keys, default_contexts_location, default_parallel_sets_location


def __upgrade(context_name, path, _all):
    if not _all:
        creds_json_ro = Bastion.load_json(Bastion.get_creds_path_resolve(context_name))
        missing_keys = set(default_config_keys).difference(creds_json_ro.keys())
        res_json = dict(creds_json_ro)
        for key in missing_keys:
            res_json[key] = "!!! New Key !!! <- This was upgraded"
        rich.print_json(data=res_json)
        if inquirer.confirm(message="Write New Json ?", default=False).execute():
            Bastion.write_creds_json(res_json, path)
            rich.print("[green]Wrote changes[/green]")
        else:
            rich.print("[red]Fill interrupted, nothing changed[/red]")

    rich.print("Searching contexts for upgrade to minimized version...")

    minimize_files = []

    for file in default_contexts_location.iterdir():
        append_minimizable(file, minimize_files)
    for set_folder in filter(lambda p: not str(p.name).startswith("."), default_parallel_sets_location.iterdir()):
        for file in set_folder.iterdir():
            if file.name.startswith(".") or not append_minimizable(file, minimize_files):
                continue
    if not len(minimize_files):
        rich.print("No minimizable files found")
        return
    if inquirer.confirm("Do you want to minimize those contexts?").execute():
        for file, data in minimize_files:
            with open(file, "w") as f:
                data.pop("ssh-pub-path", "")
                data.pop("private-key-path", "")
                rich.print(f"Minimizing {f.name}...")
                json.dump(data, f, indent=4)


def append_minimizable(file, minimize_files) -> bool:
    conf = Bastion.load_config()
    with open(file, "r") as f:
        try:
            data = json.load(f)
            if type(data) is not dict:
                return False
        except (JSONDecodeError, UnicodeDecodeError):
            return False
        if Path(data.get("ssh-pub-path", "")).name == Path(conf.get("ssh-pub-path")).name or \
                Path(data.get("private-key-path", "")).name == Path(conf.get("private-key-path")).name:
            rich.print(f" > Minimizable context {file}")
            minimize_files.append((file, data))
            return True
